#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2013, 2014 Martin Raspaud

# Author(s):

#   Martin Raspaud <martin.raspaud@smhi.se>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Some generalized spherical functions.

base type is a numpy array of size (n, 2) (2 for lon and lats)

"""

import numpy as np
import logging

logger = logging.getLogger(__name__)


class SCoordinate(object):

    """Spherical coordinates
    """

    def __init__(self, lon, lat):
        self.lon = lon
        self.lat = lat

    def cross2cart(self, point):
        """Compute the cross product, and convert to cartesian coordinates
        """

        lat1 = self.lat
        lon1 = self.lon
        lat2 = point.lat
        lon2 = point.lon

        ad = np.sin(lat1 - lat2) * np.cos((lon1 - lon2) / 2.0)
        be = np.sin(lat1 + lat2) * np.sin((lon1 - lon2) / 2.0)
        c = np.sin((lon1 + lon2) / 2.0)
        f = np.cos((lon1 + lon2) / 2.0)
        g = np.cos(lat1)
        h = np.cos(lat2)
        i = np.sin(lon2 - lon1)
        res = CCoordinate(np.array([-ad * c + be * f,
                                    ad * f + be * c,
                                    g * h * i]))

        return res

    def to_cart(self):
        """Convert to cartesian.
        """
        return CCoordinate(np.array([np.cos(self.lat) * np.cos(self.lon),
                                     np.cos(self.lat) * np.sin(self.lon),
                                     np.sin(self.lat)]))

    def distance(self, point):
        """Vincenty formula.
        """

        dlambda = self.lon - point.lon
        num = ((np.cos(point.lat) * np.sin(dlambda)) ** 2 +
               (np.cos(self.lat) * np.sin(point.lat) -
                np.sin(self.lat) * np.cos(point.lat) *
                np.cos(dlambda)) ** 2)
        den = (np.sin(self.lat) * np.sin(point.lat) +
               np.cos(self.lat) * np.cos(point.lat) * np.cos(dlambda))

        return np.arctan2(num ** .5, den)

    def hdistance(self, point):
        """Haversine formula
        """

        return 2 * np.arcsin((np.sin((point.lat - self.lat) / 2.0) ** 2.0 +
                              np.cos(point.lat) * np.cos(self.lat) *
                              np.sin((point.lon - self.lon) / 2.0) ** 2.0) ** .5)

    def __ne__(self, other):
        return not self.__eq__(other)

    def __eq__(self, other):
        return np.allclose((self.lon, self.lat), (other.lon, other.lat))

    def __str__(self):
        return str((np.rad2deg(self.lon), np.rad2deg(self.lat)))

    def __repr__(self):
        return str((np.rad2deg(self.lon), np.rad2deg(self.lat)))

    def __iter__(self):
        return [self.lon, self.lat].__iter__()


class CCoordinate(object):

    """Cartesian coordinates
    """

    def __init__(self, cart):
        self.cart = cart

    def norm(self):
        """Euclidean norm of the vector.
        """
        return np.sqrt(np.einsum('...i, ...i', self.cart, self.cart))

    def normalize(self):
        """normalize the vector.
        """

        self.cart /= np.sqrt(np.einsum('...i, ...i', self.cart, self.cart))

        return self

    def cross(self, point):
        """cross product with another vector.
        """
        return CCoordinate(np.cross(self.cart, point.cart))

    def dot(self, point):
        """dot product with another vector.
        """
        return np.inner(self.cart, point.cart)

    def __ne__(self, other):
        return not self.__eq__(other)

    def __eq__(self, other):
        return np.allclose(self.cart, other.cart)

    def __str__(self):
        return str(self.cart)

    def __add__(self, other):
        return CCoordinate(self.cart + other)

    def __radd__(self, other):
        return CCoordinate(self.cart + other)

    def __mul__(self, other):
        return CCoordinate(self.cart * other)

    def __rmul__(self, other):
        return CCoordinate(self.cart * other)

    def to_spherical(self):
        return SCoordinate(np.arctan2(self.cart[1], self.cart[0]),
                           np.arcsin(self.cart[2]))

EPSILON = 0.0000001


def modpi(val, mod=np.pi):
    """Puts *val* between -pi and pi.
    """
    return (val + mod) % (2 * mod) - mod


class Arc(object):

    """An arc of the great circle between two points.
    """
    start = None
    end = None

    def __init__(self, start, end):
        self.start, self.end = start, end

    def __eq__(self, other):
        if(self.start == other.start and self.end == other.end):
            return 1
        return 0

    def __ne__(self, other):
        return not self.__eq__(other)

    def __str__(self):
        return (str(self.start) + " -> " + str(self.end))

    def __repr__(self):
        return (str(self.start) + " -> " + str(self.end))

    def angle(self, other_arc):
        """Oriented angle between two arcs.
        """
        if self.start == other_arc.start:
            a__ = self.start
            b__ = self.end
            c__ = other_arc.end
        elif self.start == other_arc.end:
            a__ = self.start
            b__ = self.end
            c__ = other_arc.start
        elif self.end == other_arc.end:
            a__ = self.end
            b__ = self.start
            c__ = other_arc.start
        elif self.end == other_arc.start:
            a__ = self.end
            b__ = self.start
            c__ = other_arc.end
        else:
            raise ValueError("No common point in angle computation.")

        ua_ = a__.cross2cart(b__)
        ub_ = a__.cross2cart(c__)

        val = ua_.dot(ub_) / (ua_.norm() * ub_.norm())
        if abs(val - 1) < EPSILON:
            angle = 0
        elif abs(val + 1) < EPSILON:
            angle = np.pi
        else:
            angle = np.arccos(val)

        n__ = ua_.normalize()
        if n__.dot(c__.to_cart()) > 0:
            return -angle
        else:
            return angle

    def intersections(self, other_arc):
        """Gives the two intersections of the greats circles defined by the 
       current arc and *other_arc*.
       From http://williams.best.vwh.net/intersect.htm
        """

        if self.end.lon - self.start.lon > np.pi:
            self.end.lon -= 2 * np.pi
        if other_arc.end.lon - other_arc.start.lon > np.pi:
            other_arc.end.lon -= 2 * np.pi
        if self.end.lon - self.start.lon < -np.pi:
            self.end.lon += 2 * np.pi
        if other_arc.end.lon - other_arc.start.lon < -np.pi:
            other_arc.end.lon += 2 * np.pi

        ea_ = self.start.cross2cart(self.end).normalize()
        eb_ = other_arc.start.cross2cart(other_arc.end).normalize()

        cross = ea_.cross(eb_)
        lat = np.arctan2(cross.cart[2],
                         np.sqrt(cross.cart[0] ** 2 + cross.cart[1] ** 2))
        lon = np.arctan2(cross.cart[1], cross.cart[0])

        return (SCoordinate(lon, lat),
                SCoordinate(modpi(lon + np.pi), -lat))

    def intersects(self, other_arc):
        """Says if two arcs defined by the current arc and the *other_arc*
        intersect. An arc is defined as the shortest tracks between two points.
        """

        return bool(self.intersection(other_arc))

    def intersection(self, other_arc):
        """Says where, if two arcs defined by the current arc and the
        *other_arc* intersect. An arc is defined as the shortest tracks between
        two points.
        """
        if self == other_arc:
            return None
        # if (self.end == other_arc.start or
        #     self.end == other_arc.end or
        #     self.start == other_arc.start or
        #     self.start == other_arc.end):
        #     return None

        for i in self.intersections(other_arc):
            a__ = self.start
            b__ = self.end
            c__ = other_arc.start
            d__ = other_arc.end

            ab_ = a__.hdistance(b__)
            cd_ = c__.hdistance(d__)

            if(((i in (a__, b__)) or
                (abs(a__.hdistance(i) + b__.hdistance(i) - ab_) < EPSILON)) and
               ((i in (c__, d__)) or
                    (abs(c__.hdistance(i) + d__.hdistance(i) - cd_) < EPSILON))):
                return i
        return None

    def get_next_intersection(self, arcs, known_inter=None):
        """Get the next intersection between the current arc and *arcs*
        """
        res = []
        for arc in arcs:
            inter = self.intersection(arc)
            if (inter is not None and
                    inter != arc.end and
                    inter != self.end):
                res.append((inter, arc))

        def dist(args):
            """distance key.
            """
            return self.start.distance(args[0])

        take_next = False
        for inter, arc in sorted(res, key=dist):
            if known_inter is not None:
                if known_inter == inter:
                    take_next = True
                elif take_next:
                    return inter, arc
            else:
                return inter, arc

        return None, None


class SphPolygon(object):

    """Spherical polygon.

    Vertices as a 2-column array of (col 1) lons and (col 2) lats is given in
    radians.
    """

    def __init__(self, vertices, radius=1):
        self.vertices = vertices
        self.lon = self.vertices[:, 0]
        self.lat = self.vertices[:, 1]
        self.radius = radius
        self.cvertices = np.array([np.cos(self.lat) * np.cos(self.lon),
                                   np.cos(self.lat) * np.sin(self.lon),
                                   np.sin(self.lat)]).T * radius
        self.x__ = self.cvertices[:, 0]
        self.y__ = self.cvertices[:, 1]
        self.z__ = self.cvertices[:, 2]

    def invert(self):
        """Invert the polygon.
        """
        self.vertices = np.flipud(self.vertices)
        self.cvertices = np.flipud(self.cvertices)
        self.lon = self.vertices[:, 0]
        self.lat = self.vertices[:, 1]
        self.x__ = self.cvertices[:, 0]
        self.y__ = self.cvertices[:, 1]
        self.z__ = self.cvertices[:, 2]

    def inverse(self):
        """Return an invesre of the polygon.
        """
        return SphPolygon(np.flipud(self.vertices))

    def aedges(self):
        """Iterator over the edges, in arcs of Coordinates.
        """
        for i in range(len(self.lon) - 1):
            yield Arc(SCoordinate(self.lon[i],
                                  self.lat[i]),
                      SCoordinate(self.lon[i + 1],
                                  self.lat[i + 1]))
        yield Arc(SCoordinate(self.lon[i + 1],
                              self.lat[i + 1]),
                  SCoordinate(self.lon[0],
                              self.lat[0]))

    def edges(self):
        """Iterator over the edges, in geographical coordinates.
        """

        for i in range(len(self.lon) - 1):
            yield (self.lon[i], self.lat[i]), (self.lon[i + 1], self.lat[i + 1])
        yield (self.lon[i + 1], self.lat[i + 1]), (self.lon[0], self.lat[0])

    def area(self):
        """Find the area of a polygon. The inside of the polygon is defined by
        having the vertices enumerated clockwise around it.

        Uses the algorithm described in [bev1987]_.

        .. [bev1987] , Michael Bevis and Greg Cambareri, "Computing the area of a spherical polygon of arbitrary shape", in *Mathematical Geology*, May 1987, Volume 19, Issue 4, pp 335-346.

        Note: The article mixes up longitudes and latitudes in equation 3! Look
        at the fortran code appendix for the correct version.
        """

        phi_a = self.lat
        phi_p = self.lat.take(np.arange(len(self.lat)) + 1, mode="wrap")
        phi_b = self.lat.take(np.arange(len(self.lat)) + 2, mode="wrap")
        lam_a = self.lon
        lam_p = self.lon.take(np.arange(len(self.lon)) + 1, mode="wrap")
        lam_b = self.lon.take(np.arange(len(self.lon)) + 2, mode="wrap")

        new_lons_a = np.arctan2(np.sin(lam_a - lam_p) * np.cos(phi_a),
                                np.sin(phi_a) * np.cos(phi_p)
                                - np.cos(phi_a) * np.sin(phi_p)
                                * np.cos(lam_a - lam_p))

        new_lons_b = np.arctan2(np.sin(lam_b - lam_p) * np.cos(phi_b),
                                np.sin(phi_b) * np.cos(phi_p)
                                - np.cos(phi_b) * np.sin(phi_p)
                                * np.cos(lam_b - lam_p))

        alpha = new_lons_a - new_lons_b
        alpha[alpha < 0] += 2 * np.pi

        return (sum(alpha) - (len(self.lon) - 2) * np.pi) * self.radius ** 2

    def _bool_oper(self, other, sign=1):
        """(Union by default)
        """

        def rotate_arcs(start_arc, arcs):
            idx = arcs.index(start_arc)
            return arcs[idx:] + arcs[:idx]

        arcs1 = [edge for edge in self.aedges()]
        arcs2 = [edge for edge in other.aedges()]

        nodes = []

        # find the first intersection, to start from.
        for edge1 in arcs1:
            inter, edge2 = edge1.get_next_intersection(arcs2)
            if inter is not None and inter != edge1.end and inter != edge2.end:
                break

        if inter is None:
            polys = [0, self, other]
            if self._is_inside(other):
                return polys[-sign]
            if other._is_inside(self):
                return polys[sign]

            return None

        while True:
            arcs1 = rotate_arcs(edge1, arcs1)
            arcs2 = rotate_arcs(edge2, arcs2)

            arc1 = Arc(inter, edge1.end)
            arc2 = Arc(inter, edge2.end)

            if np.sign(arc1.angle(arc2)) != sign:
                arcs1, arcs2 = arcs2, arcs1

            nodes.append(inter)

            for edge1 in arcs1:
                inter, edge2 = edge1.get_next_intersection(arcs2, inter)

                if inter is not None:
                    break
                elif len(nodes) > 0 and nodes[-1] != edge1.end:
                    nodes.append(edge1.end)

            if inter is None and len(nodes) > 2 and nodes[-1] == nodes[0]:
                nodes = nodes[:-1]
                break
            if inter == nodes[0]:
                break

        return SphPolygon(np.array([(node.lon, node.lat) for node in nodes]))

    def union(self, other):
        return self._bool_oper(other, 1)

    def intersection(self, other):
        return self._bool_oper(other, -1)

    def _is_inside(self, other):
        """Checks if the polygon is entirely inside the other. Should be use
        with :meth:`inter` first to check if the is a known intersection.
        """
        # This one has no intersections
        # arc = Arc(SCoordinate(self.lon[0],
        #                       self.lat[0]),
        #           SCoordinate(self.lon[1],
        #                       self.lat[1]))

        anti_lon_0 = self.lon[0] + np.pi
        if anti_lon_0 > np.pi:
            anti_lon_0 -= np.pi * 2

        anti_lon_1 = self.lon[1] + np.pi
        if anti_lon_1 > np.pi:
            anti_lon_1 -= np.pi * 2

        arc1 = Arc(SCoordinate(self.lon[1],
                               self.lat[1]),
                   SCoordinate(anti_lon_0,
                               -self.lat[0]))

        arc2 = Arc(SCoordinate(anti_lon_0,
                               -self.lat[0]),
                   SCoordinate(anti_lon_1,
                               -self.lat[1]))

        arc3 = Arc(SCoordinate(anti_lon_1,
                               -self.lat[1]),
                   SCoordinate(self.lon[0],
                               self.lon[0]))

        other_arcs = [edge for edge in other.aedges()]
        for arc in [arc1, arc2, arc3]:
            inter, other_arc = arc.get_next_intersection(other_arcs)
            if inter is not None:
                sarc = Arc(arc.start, inter)
                earc = Arc(inter, other_arc.end)
                return sarc.angle(earc) < 0
        return other.area() > (2 * np.pi * other.radius ** 2)

    def draw(self, mapper, options):
        lons = np.rad2deg(self.lon.take(np.arange(len(self.lon) + 1),
                                        mode="wrap"))
        lats = np.rad2deg(self.lat.take(np.arange(len(self.lat) + 1),
                                        mode="wrap"))
        rx, ry = mapper(lons, lats)
        mapper.plot(rx, ry, options)

    def __str__(self):
        return str(np.rad2deg(self.vertices))


def get_twilight_poly(utctime):
    """Return a polygon enclosing the sunlit part of the globe at *utctime*.
    """
    from pyorbital import astronomy
    ra, dec = astronomy.sun_ra_dec(utctime)
    lon = modpi(ra - astronomy.gmst(utctime))
    lat = dec

    vertices = np.zeros((4, 2))

    vertices[0, :] = modpi(lon - np.pi / 2), 0
    if lat <= 0:
        vertices[1, :] = lon, np.pi / 2 + lat
        vertices[3, :] = modpi(lon + np.pi), -(np.pi / 2 + lat)
    else:
        vertices[1, :] = modpi(lon + np.pi), np.pi / 2 - lat
        vertices[3, :] = lon, -(np.pi / 2 - lat)

    vertices[2, :] = modpi(lon + np.pi / 2), 0

    return SphPolygon(vertices)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2014 Martin Raspaud

# Author(s):

#   Martin Raspaud <martin.raspaud@smhi.se>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""get dump schedule from ftp.
"""

HOST = "ftp://is.sci.gsfc.nasa.gov/ancillary/ephemeris/schedule/aqua/downlink/"

import ftplib
import urlparse
import socket
import logging
from datetime import datetime
import os

logger = logging.getLogger(__name__)

def process_line(line):
    station, aos, elev, los = line.split()[:4]
    aos = datetime.strptime(aos, "%Y:%j:%H:%M:%S")
    los = datetime.strptime(los, "%Y:%j:%H:%M:%S")
#def main():
if __name__ == '__main__':
    url = urlparse.urlparse(HOST)
    try:
        f = ftplib.FTP(url.netloc)
    except (socket.error, socket.gaierror), e:
        logger.error('cannot reach to %s ' % HOST + str(e))
        #return

    logger.debug("Connect to ftp server")

    try:
        f.login('anonymous','guest')
    except ftplib.error_perm:
        logger.error('cannot login anonymously')
        f.quit()
        #return
    logger.debug("logged on to the ftp server")

    data = []

    f.dir(url.path, data.append)
    
    filenames = [line.split()[-1] for line in data]
    dates = [datetime.strptime("".join(filename.split(".")[2:4]), "%Y%j%H%M%S") for filename in filenames]
    filedates = dict(zip(dates, filenames))

    for date in sorted(dates, reverse=True):
        lines = []
        f.retrlines('RETR ' + os.path.join(url.path, filedates[date]), lines.append)

        for line in lines[7::2]:
            if line == '':
                break
            
        
        pause

    
    
#if __name__ == '__main__':
#    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2014 Martin Raspaud

# Author(s):

#   Martin Raspaud <martin.raspaud@smhi.se>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Satellite passes.
"""

import logging
import logging.handlers
import os
import operator
from datetime import datetime, timedelta
import numpy as np
from pyorbital import tlefile, orbital
from tempfile import mkstemp
from trollsched.boundary import SwathBoundary, AreaDefBoundary
import glob

logger = logging.getLogger(__name__)

# shortest allowed pass in minutes
MIN_PASS = 4


class Mapper(object):

    """A class to generate nice plots with basemap.
    """

    def __init__(self):
        from mpl_toolkits.basemap import Basemap

        self.map = Basemap(projection='nsper', lat_0=20, lon_0=40,
                           resolution='l', area_thresh=1000.)

        self.map.drawcoastlines()
        self.map.drawcountries()

        self.map.drawmapboundary(fill_color='white')

        self.map.drawmeridians(np.arange(0, 360, 30))
        self.map.drawparallels(np.arange(-90, 90, 30))

    def __enter__(self):
        return self.map

    def __exit__(self, etype, value, tb):
        pass

# Platform name to tle names translator
tle_names = {"Metop-A": "METOP-A",
             "Metop-B": "METOP-B",
             "NOAA-15": "NOAA 15",
             "NOAA-18": "NOAA 18",
             "NOAA-19": "NOAA 19",
             "Suomi-NPP": "SUOMI NPP",
             "EOS-Terra": "TERRA",
             "EOS-Aqua": "AQUA"}


class Pass(object):

    """A pass: satellite, risetime, falltime, (orbital)
    """

    buffer = timedelta(minutes=2)

    def __init__(self, satellite, risetime, falltime, orb=None, uptime=None, instrument=None):

        self.satellite = satellite
        self.risetime = risetime
        self.falltime = falltime
        self.uptime = uptime
        self.instrument = instrument
        self.orb = orb or orbital.Orbital(tle_names.get(satellite,
                                                        satellite))
        self.score = {}
        self.boundary = SwathBoundary(self)
        # make boundary lighter.
        # self.boundary.decimate(100)
        self.subsattrack = {"start": None,
                            "end": None}
        self.rec = False
        self.fig = None

    def overlaps(self, other, delay=timedelta(seconds=0)):
        """Check if two passes overlap in time.
        """
        return ((self.risetime < other.falltime + delay) and
                (self.falltime + delay > other.risetime))

    def __cmp__(self, other):
        if self.uptime < other.uptime:
            return -1
        if self.uptime > other.uptime:
            return 1
        else:
            return 0

    def __eq__(self, other):
        return (self.risetime == other.risetime and
                self.falltime == other.falltime and
                self.satellite == other.satellite)

    def __str__(self):
        return (self.satellite + " "
                + self.risetime.isoformat() + " " + self.falltime.isoformat())

    def __repr__(self):
        return str(self)

    def duration(self):
        """Get the duration of an overpass.
        """
        return self.falltime - self.risetime

    def seconds(self):
        """Get the duration of an overpass.
        """
        duration = self.duration()
        return (duration.days * 24 * 60 * 60
                + duration.seconds
                + duration.microseconds * 1e-6)

    def pass_direction(self):
        """Get the direction of the pass in (ascending, descending).
        """
        start_lat = self.orb.get_lonlatalt(self.risetime)[1]
        end_lat = self.orb.get_lonlatalt(self.falltime)[1]

        if start_lat > end_lat:
            return "descending"
        else:
            return "ascending"

    def slsearch(self, sublat):
        """Find sublatitude.
        """

        def nadirlat(minutes):
            return self.orb.get_lonlatalt(self.risetime +
                                          timedelta(minutes=np.float64(minutes)))[1] - sublat

        def get_root(fun, start, end):
            p = np.polyfit([start, (start + end) / 2.0, end],
                           [fun(start), fun((start + end) / 2), fun(end)],
                           2)
            for root in np.roots(p):
                if root <= end and root >= start:
                    return root

        arr = np.array([nadirlat(m) for m in range(15)])
        a = np.where(np.diff(np.sign(arr)))[0]
        for guess in a:
            sublat_mins = get_root(nadirlat, guess, guess + 1)
            return self.risetime + timedelta(minutes=sublat_mins)

    def area_coverage(self, area_of_interest):
        """Get the ratio of coverage (between 0 and 1) of the pass with the area
        of interest.
        """
        try:
            area_boundary = area_of_interest.poly
        except AttributeError:
            area_boundary = AreaDefBoundary(area_of_interest,
                                            frequency=500)
            area_boundary = area_boundary.contour_poly
        inter = self.boundary.contour_poly.intersection(
            area_boundary)
        if inter is None:
            return 0
        return inter.area() / area_boundary.area()

    def save_fig(self, poly=None, directory="/tmp/plots",
                 overwrite=False, labels=None, extension=".png"):
        """Save the pass as a figure. Filename is automatically generated.
        """
        logger.debug("Save fig " + str(self))
        rise = self.risetime.strftime("%Y%m%d%H%M%S")
        fall = self.falltime.strftime("%Y%m%d%H%M%S")
        filename = os.path.join(directory,
                                (rise + self.satellite + fall + extension))

        self.fig = filename
        if not overwrite and os.path.exists(filename):
            return filename

        import matplotlib.pyplot as plt
        plt.clf()
        with Mapper() as mapper:
            mapper.nightshade(self.uptime, alpha=0.2)
            self.draw(mapper, "-r")
            if poly is not None:
                poly.draw(mapper, "-b")
        plt.title(str(self))
        for label in labels or []:
            plt.figtext(*label[0], **label[1])
        plt.savefig(filename)
        return filename

    def show(self, poly=None, labels=None, other_poly=None):
        """Show the current pass on screen (matplotlib, basemap).
        """
        import matplotlib.pyplot as plt
        plt.clf()
        with Mapper() as mapper:
            mapper.nightshade(self.uptime, alpha=0.2)
            self.draw(mapper, "-r")
            if poly is not None:
                poly.draw(mapper, "-b")
            if other_poly is not None:
                other_poly.draw(mapper, "-g")
        plt.title(str(self))
        for label in (labels or []):
            plt.figtext(*label[0], **label[1])
        plt.show()

    def draw(self, mapper, options):
        """Draw the pass to the *mapper* object (basemap).
        """
        self.boundary.contour_poly.draw(mapper, options)

    def print_vcs(self, coords):
        """Should look like this::


#SCName          RevNum Risetime        Falltime        Elev Dura ANL   Rec Dir Man Ovl OvlSCName        OvlRev OvlRisetime     OrigRisetime    OrigFalltime    OrigDuration
#
NOAA 19           24845 20131204 001450 20131204 003003 32.0 15.2 225.6 Y   Des N   N   none                  0 19580101 000000 20131204 001450 20131204 003003 15.2


        """

        max_elevation = self.orb.get_observer_look(self.uptime, *coords)[1]
        anl = self.orb.get_lonlatalt(
            self.orb.get_last_an_time(self.risetime))[0] % 360
        #anl = self.orb.get_observer_look(self.risetime, *coords)[0]
        if self.rec:
            rec = "Y"
        else:
            rec = "N"
        line_list = ["{satellite:<16}",
                     "{orbit:>6}",
                     "{risetime}",
                     "{falltime}",
                     "{elevation:>4.1f}",
                     "{duration:>4.1f}",
                     "{anl:>5.1f}",
                     "{rec:<3}",
                     "{direction}",
                     "N   N   none                  0 19580101 000000",
                     "{risetime}",
                     "{falltime}",
                     "{duration:>4.1f}",
                     ]
        line = " ".join(line_list).format(
            satellite=self.satellite.upper(),
            orbit=self.orb.get_orbit_number(self.risetime),
            risetime=self.risetime.strftime("%Y%m%d %H%M%S"),
            falltime=self.falltime.strftime("%Y%m%d %H%M%S"),
            elevation=max_elevation,
            duration=(self.falltime - self.risetime).seconds / 60.0,
            anl=anl,
            rec=rec,
            direction=self.pass_direction().capitalize()[:3])
        return line

HOST = "ftp://is.sci.gsfc.nasa.gov/ancillary/ephemeris/schedule/aqua/downlink/"
import urlparse
import ftplib
import socket


def get_aqua_dumps_from_ftp(start_time, end_time, satorb):
    url = urlparse.urlparse(HOST)
    logger.debug("Connect to ftp server")
    try:
        f = ftplib.FTP(url.netloc)
    except (socket.error, socket.gaierror), e:
        logger.error('cannot reach to %s ' % HOST + str(e))
        f = None

    if f is not None:
        try:
            f.login('anonymous', 'guest')
            logger.debug("Logged in")
        except ftplib.error_perm:
            logger.error('cannot login anonymously')
            f.quit()
            f = None

    if f is not None:
        data = []
        try:
            f.dir(url.path, data.append)
        except socket.error, e:
            logger.error("Can't get any data: " + str(e))
            f.quit()
            f = None
        else:
            filenames = [line.split()[-1] for line in data]

    if f is None:
        logger.info("Can't access ftp server, using cached data")
        filenames = glob.glob("/tmp/*.rpt")

    dates = [datetime.strptime("".join(filename.split(".")[2:4]), "%Y%j%H%M%S")
             for filename in filenames]
    filedates = dict(zip(dates, filenames))

    dumps = []

    for date in sorted(dates):
        lines = []
        if not filedates[date].endswith(".rpt"):
            continue
        if not os.path.exists(os.path.join("/tmp", filedates[date])):
            f.retrlines(
                'RETR ' + os.path.join(url.path, filedates[date]), lines.append)
            with open(os.path.join("/tmp", filedates[date]), "w") as fd_:
                for line in lines:
                    fd_.write(line + "\n")
        else:
            with open(os.path.join("/tmp", filedates[date]), "r") as fd_:
                for line in fd_:
                    lines.append(line)

        for line in lines[7::2]:
            if line.strip() == '':
                break
            station, aos, elev, los = line.split()[:4]
            aos = datetime.strptime(aos, "%Y:%j:%H:%M:%S")
            los = datetime.strptime(los, "%Y:%j:%H:%M:%S")
            if los >= start_time and aos <= end_time:
                uptime = aos + (los - aos) / 2
                overpass = Pass("aqua", aos, los, satorb, uptime, "modis")
                overpass.station = station
                overpass.max_elev = elev
                dumps.append(overpass)
    if f is not None:
        f.quit()
    return dumps


def get_next_passes(satellites, utctime, forward, coords, tle_file=None):
    """Get the next passes for *satellites*, starting at *utctime*, for a
    duration of *forward* hours, with observer at *coords* ie lon (°E), lat
    (°N), altitude (km). Uses *tle_file* if provided, downloads from celestrack
    otherwise.
    """
    passes = {}
    orbitals = {}

    if tle_file is None:
        fp_, tle_file = mkstemp(prefix="tle", dir="/tmp")
        os.close(fp_)
        logger.info("Fetch tle info from internet")
        tlefile.fetch(tle_file)

    if not os.path.exists(tle_file):
        logger.info("Fetch tle info from internet")
        tlefile.fetch(tle_file)

    for sat in satellites:
        satorb = orbital.Orbital(sat, tle_file=tle_file)
        orbitals[sat] = satorb
        passlist = satorb.get_next_passes(utctime,
                                          forward,
                                          *coords)
        if sat.startswith("metop") or sat.startswith("noaa"):
            instrument = "avhrr"
        elif sat in ["aqua", "terra"]:
            instrument = "modis"
        elif sat.endswith("npp"):
            instrument = "viirs"
        else:
            instrument = "unknown"
        # take care of metop-a
        if sat == "metop-a":
            metop_passes = [Pass(sat, rtime, ftime, satorb, uptime, instrument)
                            for rtime, ftime, uptime in passlist]

            passes["metop-a"] = []
            for overpass in metop_passes:
                if overpass.pass_direction() == "descending":
                    new_rise = overpass.slsearch(60)
                    if new_rise is not None and new_rise < overpass.falltime:
                        overpass.risetime = new_rise
                        overpass.boundary = SwathBoundary(overpass)
                        if overpass.seconds() > MIN_PASS * 60:
                            passes["metop-a"].append(overpass)
        # take care of aqua (dumps in svalbard and poker flat)
        elif sat == "aqua":

            wpcoords = (-75.457222, 37.938611, 0)
            passlist_wp = satorb.get_next_passes(utctime - timedelta(minutes=30),
                                                 forward + 1,
                                                 *wpcoords)
            wp_passes = [Pass(sat, rtime, ftime, satorb, uptime, instrument)
                         for rtime, ftime, uptime in passlist_wp]

            svcoords = (15.399, 78.228, 0)
            passlist_sv = satorb.get_next_passes(utctime - timedelta(minutes=30),
                                                 forward + 1,
                                                 *svcoords)
            sv_passes = [Pass(sat, rtime, ftime, satorb, uptime, instrument)
                         for rtime, ftime, uptime in passlist_sv]
            pfcoords = (-147.43, 65.12, 0.51)
            passlist_pf = satorb.get_next_passes(utctime - timedelta(minutes=30),
                                                 forward + 1,
                                                 *pfcoords)
            pf_passes = [Pass(sat, rtime, ftime, satorb, uptime, instrument)
                         for rtime, ftime, uptime in passlist_pf]

            aqua_passes = [Pass(sat, rtime, ftime, satorb, uptime, instrument)
                           for rtime, ftime, uptime in passlist]

            dumps = get_aqua_dumps_from_ftp(utctime - timedelta(minutes=30),
                                            utctime +
                                            timedelta(hours=forward + 0.5),
                                            satorb)

            # remove the known dumps
            for dump in dumps:
                # print "*", dump.station, dump, dump.max_elev
                logger.debug("dump from ftp: " + str((dump.station, dump,
                                                      dump.max_elev)))
                for i, sv_pass in enumerate(sv_passes):
                    if sv_pass.overlaps(dump, timedelta(minutes=40)):
                        sv_elevation = sv_pass.orb.get_observer_look(sv_pass.uptime,
                                                                     *svcoords)[1]
                        logger.debug("Computed " + str(("SG", sv_pass,
                                                        sv_elevation)))
                        del sv_passes[i]
                for i, pf_pass in enumerate(pf_passes):
                    if pf_pass.overlaps(dump, timedelta(minutes=40)):
                        pf_elevation = pf_pass.orb.get_observer_look(pf_pass.uptime,
                                                                     *pfcoords)[1]
                        logger.debug("Computed " + str(("PF", pf_pass,
                                                        pf_elevation)))
                        del pf_passes[i]
                for i, wp_pass in enumerate(wp_passes):
                    if wp_pass.overlaps(dump, timedelta(minutes=40)):
                        wp_elevation = wp_pass.orb.get_observer_look(wp_pass.uptime,
                                                                     *wpcoords)[1]
                        logger.debug("Computed " + str(("WP", wp_pass,
                                                        wp_elevation)))
                        del wp_passes[i]

            # sort out dump passes first
            # between sv an pf, we take the one with the highest elevation if
            # pf < 20°, pf otherwise
            # I think wp is also used if sv is the only other alternative
            used_pf = []
            for sv_pass in sv_passes:
                found_pass = False
                for pf_pass in pf_passes:
                    if sv_pass.overlaps(pf_pass):
                        found_pass = True
                        used_pf.append(pf_pass)
                        sv_elevation = sv_pass.orb.get_observer_look(sv_pass.uptime,
                                                                     *svcoords)[1]
                        pf_elevation = pf_pass.orb.get_observer_look(pf_pass.uptime,
                                                                     *pfcoords)[1]
                        if pf_elevation > 20:
                            dumps.append(pf_pass)
                        elif sv_elevation > pf_elevation:
                            dumps.append(sv_pass)
                        else:
                            dumps.append(pf_pass)
                        break
                if not found_pass:
                    dumps.append(sv_pass)

            for pf_pass in pf_passes:
                if pf_pass not in used_pf:
                    dumps.append(pf_pass)

            passes["aqua"] = []
            for overpass in aqua_passes:
                add = True
                for dump_pass in dumps:
                    if dump_pass.overlaps(overpass):
                        if (dump_pass.uptime < overpass.uptime and
                                dump_pass.falltime > overpass.risetime):
                            logger.debug("adjusting " + str(overpass)
                                         + " to new risetime " +
                                         str(dump_pass.falltime))
                            overpass.risetime = dump_pass.falltime
                            overpass.boundary = SwathBoundary(overpass)
                        elif (dump_pass.uptime >= overpass.uptime and
                              dump_pass.risetime < overpass.falltime):
                            logger.debug("adjusting " + str(overpass)
                                         + " to new falltime " +
                                         str(dump_pass.risetime))
                            overpass.falltime = dump_pass.risetime
                            overpass.boundary = SwathBoundary(overpass)
                        if overpass.falltime <= overpass.risetime:
                            add = False
                            logger.debug("skipping " + str(overpass))
                if add and overpass.seconds() > MIN_PASS * 60:
                    passes["aqua"].append(overpass)

        else:
            passes[sat] = [Pass(sat, rtime, ftime, satorb, uptime, instrument)
                           for rtime, ftime, uptime in passlist
                           if ftime - rtime > timedelta(minutes=MIN_PASS)]

    return set(reduce(operator.concat, passes.values()))

if __name__ == '__main__':
    from trollsched.satpass import get_next_passes
    passes = get_next_passes(
        ["noaa 19", "suomi npp"], datetime.now(), 24, (16, 58, 0))
    for p in passes:
        p.save_fig(directory="/tmp/plots/")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2014 Martin Raspaud

# Author(s):

#   Martin Raspaud <martin.raspaud@smhi.se>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Compare the request file and the confirmation file.
"""

import logging
import logging.handlers
import sys
import os
import glob

logger = logging.getLogger(__name__)

def xml_compare(x1_, x2_, reporter=None, skiptags=None):
    """Compare xml objects.
    """
    if x1_.tag != x2_.tag:
        if reporter:
            reporter('Tags do not match: %s and %s' % (x1_.tag, x2_.tag))
        return False
    for name, value in x1_.attrib.items():
        if x2_.attrib.get(name) != value:
            if reporter:
                reporter('Attributes do not match: %s=%r, %s=%r'
                         % (name, value, name, x2_.attrib.get(name)))
            return False
    for name in x2_.attrib.keys():
        if name not in x1_.attrib:
            if reporter:
                reporter('x2_ has an attribute x1_ is missing: %s'
                         % name)
            return False
    if not text_compare(x1_.text, x2_.text):
        if reporter:
            reporter('text: %r != %r' % (x1_.text, x2_.text))
        return False
    if not text_compare(x1_.tail, x2_.tail):
        if reporter:
            reporter('tail: %r != %r' % (x1_.tail, x2_.tail))
        return False
    cl1 = x1_.getchildren()
    cl2 = x2_.getchildren()
    if len(cl1) != len(cl2):
        if reporter:
            reporter('not the same number of passes, %i != %i'
                     % (len(cl1), len(cl2)))
        return False
    i = 0
    for c1, c2 in zip(cl1, cl2):
        i += 1
        if skiptags and (c1.tag in skiptags):
            continue
        if not xml_compare(c1, c2, reporter=reporter):
            if reporter:
                reporter('element %i do not match: %s'
                         % (i, c1.tag))
            return False
    return True


def text_compare(t1_, t2_):
    """Compare text fields.
    """
    if not t1_ and not t2_:
        return True
    if t1_ == '*' or t2_ == '*':
        return True
    return (t1_ or '').strip() == (t2_ or '').strip()

def compare(file1, file2):
    """Compare two xml files, request and confirmation.
    """
    import xml.etree.ElementTree as ET
    xml1 = ET.parse(file1).getroot()
    xml2 = ET.parse(file2).getroot()
    if xml_compare(xml1, xml2, logger.error,
                   ["confirmed-by", "confirmed-on", "properties"]):
        logger.info("All passes confirmed.")
        return True
    else:
        return False


# import fnmatch
# import pyinotify

# class EventHandler(pyinotify.ProcessEvent):
#     """Manage events.
#     """
#     def process_IN_CLOSE_WRITE(self, event):
#         if not fnmatch.fnmatch(event.pathname, "*-acquisition-schedule-confirmation-???.xml"):
#             return
#         logger.info("Processing: %s", event.pathname)
#         reqname = event.pathname[:-20] + "request" +  event.pathname[-8:]
#         logger.info("Validating against: %s", reqname)
#         compare(reqname, event.pathname)
#         sys.exit(0)

#     def process_IN_MOVED_TO(self, event):
#         self.process_IN_CLOSE_WRITE(event)

def run():
    import argparse

    parser = argparse.ArgumentParser()
    parser.add_argument("-f", "--file", help="req/conf files to compare",
                        nargs=2)

    parser.add_argument("-m", "--mail", nargs="*",
                        help="mail address(es) to send error messages to.",
                        default=None)
    parser.add_argument("-v", "--verbose", help="activate debug messages",
                        action="store_true")
    parser.add_argument("-l", "--log", help="file to log to")
#    parser.add_argument("-w", "--watch",
#                        help="directory to watch for new confirmation files")
    parser.add_argument("-r", "--most-recent",
                        help="check the most recent request against the" +
                        " corresponding confirmation, from the given directory")
    parser.add_argument("-c", "--confirmation",
                        help="directory for the confirmation files")
    
    opts = parser.parse_args()

    if opts.log:
        handler = logging.handlers.TimedRotatingFileHandler(opts.log,
                                                            "midnight",
                                                            backupCount=7)
    else:
        handler = logging.StreamHandler()
    handler.setFormatter(logging.Formatter("[%(levelname)s: %(asctime)s :"
                                           " %(name)s] %(message)s",
                                           '%Y-%m-%d %H:%M:%S'))

    if opts.verbose:
        loglevel = logging.DEBUG
    else:
        loglevel = logging.INFO
    handler.setLevel(loglevel)
    logging.getLogger('').setLevel(loglevel)
    logging.getLogger('').addHandler(handler)


    if opts.mail:
        mhandler = logging.handlers.SMTPHandler("localhost",
                                                "satsateknik@smhi.se",
                                                opts.mail,
                                                "Scheduler confirmation")
        mhandler.setLevel(logging.WARNING)
        logging.getLogger('').addHandler(mhandler)
    logger = logging.getLogger("compare")

    logger.debug("DEBUG on")

    if opts.file:
        compare(opts.file[0], opts.file[1])

    # if opts.watch:
    #     wm = pyinotify.WatchManager() # Watch Manager
    #     mask = pyinotify.IN_CLOSE_WRITE | pyinotify.IN_MOVED_TO
    #     handler = EventHandler()
    #     notifier = pyinotify.Notifier(wm, handler)
    #     wdd = wm.add_watch(opts.watch, mask, rec=False)

    #     notifier.loop()



    if opts.most_recent:
        logger.debug("looking for most recent file in " +
                     os.path.join(opts.most_recent, "*request*.xml"))
        filelist = glob.glob(os.path.join(opts.most_recent, "*request*.xml"))
        newest = max(filelist, key=lambda x: os.stat(x).st_mtime)
        logger.debug("checking " + newest)
        reqdir, newfile = os.path.split(newest)
        confdir = opts.confirmation or reqdir
        confname = os.path.join(confdir,
                                newfile[:-15] + "confirmation" +  newfile[-8:])
        logger.debug("against " + confname)
        try:
            compare(newest, confname)
        except IOError:
            logger.exception("Something went wrong!") 

if __name__ == '__main__':
    run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2013, 2014 Martin Raspaud

# Author(s):

#   Martin Raspaud <martin.raspaud@smhi.se>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Scheduling
"""
import logging
import logging.handlers
import urlparse
import os
from datetime import datetime, timedelta
from pprint import pformat
import numpy as np
from pyresample import utils
from pyorbital import astronomy
from trollsched.spherical import get_twilight_poly
from trollsched.graph import Graph
from trollsched.satpass import get_next_passes
from trollsched.boundary import AreaDefBoundary

from ConfigParser import ConfigParser

logger = logging.getLogger(__name__)

# shortest allowed pass in minutes
MIN_PASS = 4


def conflicting_passes(allpasses, delay=timedelta(seconds=0)):
    """Get the passes in groups of conflicting passes.
    """

    passes = sorted(allpasses, key=lambda x: x.risetime)

    overpass = passes[0]
    last_time = overpass.falltime
    group = [overpass]
    groups = []
    for overpass in passes[1:]:
        if overpass.risetime - delay < last_time:
            group.append(overpass)
            if last_time < overpass.falltime:
                last_time = overpass.falltime

        else:
            groups.append(group)
            group = [overpass]
            last_time = overpass.falltime

    groups.append(group)
    return groups


def get_non_conflicting_groups(passes, delay=timedelta(seconds=0)):
    """Get the different non-conflicting solutions in a group of conflicting
    passes.
    """
    # Uses graphs and maximal clique finding with the Bron-Kerbosch algorithm.

    order = len(passes)

    if order == 1:
        return [passes]

    graph = Graph(order)

    for i, overpass in enumerate(sorted(passes, key=lambda x: x.risetime)):
        for j in range(i + 1, order):
            if not overpass.overlaps(passes[j], delay):
                graph.add_edge(i, j)

    groups = []
    for res in graph.bron_kerbosch(set(), set(graph.vertices), set()):
        grp = []
        for vertex in res:
            grp.append(passes[vertex])
        groups.append(sorted(grp))

    return groups


def fermia(t):
    a = 0.25
    b = a / 4
    k = b * np.log(1 / 3.0) + a
    sh = k - 0.25
    return 0.5 / (np.exp(((t + sh) - a) / b) + 1) + 0.5


def fermib(t):
    a = 0.25
    b = a / 4
    return 1 / (np.exp((t - a) / b) + 1)

combination = {}


def combine(p1, p2, area_of_interest, scores):
    """Combine passes together.
    """

    try:
        return combination[p1, p2]
    except KeyError:
        pass

    area = area_of_interest.poly.area()

    def pscore(poly, coeff=1):
        if poly is None:
            return 0
        else:
            return poly.area() * coeff

    twi1 = get_twilight_poly(p1.uptime)
    twi2 = get_twilight_poly(p2.uptime)

    ip1, sip1 = p1.score.get(area_of_interest, (None, None))
    if sip1 is None:
        ip1 = p1.boundary.contour_poly.intersection(area_of_interest.poly)
        # FIXME: ip1 or ip2 could be None if the pass is entirely inside the
        # area (or vice versa)
        if ip1 is None:
            return 0

        ip1d = ip1.intersection(twi1)
        if ip1d is None:
            lon, lat = np.rad2deg(ip1.vertices[0, :])
            theta = astronomy.cos_zen(p1.uptime,
                                      lon, lat)
            if np.sign(theta) > 0:
                ip1d = ip1
                ip1n = None
            else:
                ip1n = ip1
        else:
            twi1.invert()
            ip1n = ip1.intersection(twi1)
            twi1.invert()

        ns1 = pscore(ip1n, scores[p1.satellite][0] / area)
        ds1 = pscore(ip1d, scores[p1.satellite][1] / area)
        sip1 = ns1 + ds1
        p1.score[area_of_interest] = (ip1, sip1)

    ip2, sip2 = p2.score.get(area_of_interest, (None, None))
    if sip2 is None:
        ip2 = p2.boundary.contour_poly.intersection(area_of_interest.poly)
        if ip2 is None:
            return 0

        ip2d = ip2.intersection(twi2)
        if ip2d is None:
            lon, lat = np.rad2deg(ip2.vertices[0, :])
            theta = astronomy.cos_zen(p2.uptime,
                                      lon, lat)
            if np.sign(theta) > 0:
                ip2d = ip2
                ip2n = None
            else:
                ip2n = ip2
        else:
            twi2.invert()
            ip2n = ip2.intersection(twi2)
            twi2.invert()

        ns2 = pscore(ip2n, scores[p2.satellite][0] / area)
        ds2 = pscore(ip2d, scores[p2.satellite][1] / area)
        sip2 = ns2 + ds2
        p2.score[area_of_interest] = (ip2, sip2)

    ip1p2 = ip1.intersection(ip2)

    if ip1p2 is None:
        sip1p2 = 0
    else:
        ip1p2da = ip1p2.intersection(twi1)
        twi1.invert()
        ip1p2na = ip1p2.intersection(twi1)
        twi1.invert()

        ip1p2db = ip1p2.intersection(twi2)
        twi2.invert()
        ip1p2nb = ip1p2.intersection(twi2)
        twi2.invert()

        ns12a = pscore(ip1p2na, scores[p1.satellite][0] / area)
        ds12a = pscore(ip1p2da, scores[p1.satellite][1] / area)
        ns12b = pscore(ip1p2nb, scores[p2.satellite][0] / area)
        ds12b = pscore(ip1p2db, scores[p2.satellite][1] / area)

        sip1p2a = ns12a + ds12a
        sip1p2b = ns12b + ds12b
        sip1p2 = (sip1p2a + sip1p2b) / 2.0

    if p2 > p1:
        tdiff = (p2.uptime - p1.uptime).seconds / 3600.
    else:
        tdiff = (p1.uptime - p2.uptime).seconds / 3600.

    res = fermia(tdiff) * (sip1 + sip2) - fermib(tdiff) * sip1p2
    combination[p1, p2] = res

    return res


def get_best_sched(overpasses, area_of_interest, scores, delay):
    """Get the best schedule based on *area_of_interest*.
    """
    passes = sorted(overpasses, key=lambda x: x.risetime)
    grs = conflicting_passes(passes, delay)
    logger.debug("conflicting %s", str(grs))
    ncgrs = [get_non_conflicting_groups(gr, delay) for gr in grs]
    logger.debug("non conflicting %s", str(ncgrs))
    n_vertices = len(passes)

    graph = Graph(n_vertices=n_vertices + 2)

    def add_arc(graph, p1, p2, hook=None):
        logger.debug("Adding arc between " + str(p1) +
                     " and " + str(p2) + "...")
        w = combine(p1, p2, area_of_interest, scores)
        logger.debug("...with weight " + str(w))

        with open("/tmp/schedule.gv", "a") as fp_:
            fp_.write('        "' + str(p1) + '" -> "' + str(p2) +
                      '" [ label = "' + str(w) + '" ];\n')

        graph.add_arc(passes.index(p1) + 1,
                      passes.index(p2) + 1, w)
        if hook is not None:
            hook()

    prev = set()
    for ncgr in ncgrs:
        for pr in prev:
            foll = set(gr[0] for gr in ncgr)
            for f in foll:
                add_arc(graph, pr, f)

        prev = set(sorted(gr, key=lambda x: x.falltime)[-1] for gr in ncgr)
        for gr in ncgr:
            if len(gr) > 1:
                for p1, p2 in zip(gr[:-1], gr[1:]):
                    add_arc(graph, p1, p2)

    for pr in prev:
        graph.add_arc(passes.index(pr) + 1, n_vertices + 1)
    for first in ncgrs[0][0]:
        graph.add_arc(0, passes.index(first) + 1)

    dist, path = graph.dag_longest_path(0, n_vertices + 1)

    del dist
    return [passes[idx - 1] for idx in path[1:-1]], (graph, passes)


def argmax(iterable):
    return max((x, i) for i, x in enumerate(iterable))[1]


def get_max(groups, fun):
    """Get the best group of *groups* using the score function *fun*
    """
    scores = []
    for grp in groups:
        scores.append(sum([fun(p) for p in grp]))
    return groups[argmax(scores)]


def generate_sch_file(output_file, overpasses, coords):

    with open(output_file, "w") as out:
        # create epochs
        out.write("#Orbital elements\n#\n#SCName           Epochtime\n#\n")
        satellites = set()

        for overpass in overpasses:
            epoch = "!{0:<16} {1}".format(overpass.satellite.upper(),
                                          overpass.orb.tle.epoch.strftime("%Y%m%d %H%M%S"))
            satellites |= set([epoch])
        sats = "\n".join(satellites) + "\n"
        out.write(sats)
        out.write("#\n#\n#Pass List\n#\n")

        out.write(
            "#SCName          RevNum Risetime        Falltime        Elev Dura ANL   Rec Dir Man Ovl OvlSCName        OvlRev OvlRisetime     OrigRisetime    OrigFalltime    OrigDuration\n#\n")

        for overpass in sorted(overpasses):
            out.write(overpass.print_vcs(coords) + "\n")


def generate_xml_requests(sched, start, end, station_name, report_mode=False):
    """Create xml requests.
    """
    import xml.etree.ElementTree as ET

    sats = {"noaa 15": "noaa15",
            "noaa 16": "noaa16",
            "noaa 18": "noaa18",
            "noaa 19": "noaa19",
            "metop-a": "metop-a",
            "metop-b": "metop-b",
            "terra": "terra",
            "aqua": "aqua",
            "suomi npp": "npp",
            }

    reqtime = datetime.utcnow()
    eum_format = "%Y-%m-%d-%H:%M:%S"

    root = ET.Element("acquisition-schedule")
    props = ET.SubElement(root, "properties")
    proj = ET.SubElement(props, "project")
    proj.text = "Pytroll"
    typep = ET.SubElement(props, "type")
    typep.text = "request"
    station = ET.SubElement(props, "station")
    station.text = station_name
    file_start = ET.SubElement(props, "file-start")
    file_start.text = start.strftime(eum_format)
    file_end = ET.SubElement(props, "file-end")
    file_end.text = end.strftime(eum_format)
    reqby = ET.SubElement(props, "requested-by")
    reqby.text = "SMHI"
    reqon = ET.SubElement(props, "requested-on")
    reqon.text = reqtime.strftime(eum_format)
    for overpass in sorted(sched):
        if (overpass.rec or report_mode) and overpass.risetime > start:
            ovpass = ET.SubElement(root, "pass")
            ovpass.set("satellite", sats.get(overpass.satellite,
                                             overpass.satellite))
            ovpass.set("start-time", overpass.risetime.strftime(eum_format))
            ovpass.set("end-time", overpass.falltime.strftime(eum_format))
            if report_mode:
                if overpass.fig is not None:
                    ovpass.set("img", overpass.fig)
                ovpass.set("rec", str(overpass.rec))

    return root, reqtime


def generate_xml_file(sched, start, end, directory, station, report_mode=False):
    """Create an xml request file.
    """
    import xml.etree.ElementTree as ET
    tree, reqtime = generate_xml_requests(sched,
                                          start, end,
                                          station, report_mode)
    if report_mode:
        mode = "report"
    else:
        mode = "request"
    filename = (reqtime.strftime("%Y-%m-%d-%H-%M-%S")
                + "-acquisition-schedule-" + mode + "-"
                + station + ".xml")
    filename = os.path.join(directory, filename)
    tmp_filename = (reqtime.strftime("%Y-%m-%d-%H-%M-%S")
                    + "-acquisition-schedule-" + mode + "-"
                    + station + ".tmp")
    tmp_filename = os.path.join(directory, tmp_filename)
    with open(tmp_filename, "w") as fp_:
        if report_mode:
            fp_.write("<?xml version='1.0' encoding='utf-8'?>"
                      "<?xml-stylesheet type='text/xsl' href='reqreader.xsl'?>")
        fp_.write(ET.tostring(tree))
    os.rename(tmp_filename, filename)
    return filename


def parse_datetime(strtime):
    """Parse the time string *strtime*
    """
    return datetime.strptime(strtime, "%Y%m%d%H%M%S")


def read_config(filename):
    """Read the config file *filename* and replace the values in global
    variables.
    """
    cfg = ConfigParser()
    cfg.read(filename)

    station = cfg.get("default", "station")
    satellites = cfg.get("default", "satellites").split(",")
    forward = cfg.getfloat("default", "forward")
    start = cfg.getfloat("default", "start")

    station_name = cfg.get(station, "name")
    station_lon = cfg.getfloat(station, "longitude")
    station_lat = cfg.getfloat(station, "latitude")
    station_alt = cfg.getfloat(station, "altitude")

    sat_scores = {}
    for sat in satellites:
        sat_scores[sat] = (cfg.getfloat(sat, "night"),
                           cfg.getfloat(sat, "day"))

    area = utils.parse_area_file(cfg.get(station, "area_file"),
                                 cfg.get(station, "area"))[0]

    return ((station_lon, station_lat, station_alt),
            sat_scores, station_name, area, forward, start)


def run():
    """The schedule command
    """
    import argparse
    global logger

    parser = argparse.ArgumentParser()
    parser.add_argument("--lon", help="Longitude, degrees east", type=float)
    parser.add_argument("--lat", help="Latitude, degrees north", type=float)
    parser.add_argument("--alt", help="Altitude, km", type=float)
    parser.add_argument("-l", "--log",
                        help="File to log to (defaults to stdout)",
                        default=None)
    parser.add_argument("-m", "--mail", nargs="*",
                        help="mail address(es) to send error messages to.",
                        default=None)
    parser.add_argument("-v", "--verbose", help="print debug messages too",
                        action="store_true")
    parser.add_argument("-g", "--graph", help="save graph info to this directory",
                        default=None)
    parser.add_argument("-t", "--tle", help="tle file to use", default=None)
    parser.add_argument("-f", "--forward", type=float,
                        help="time ahead to compute the schedule")
    parser.add_argument("-s", "--start-time",
                        type=parse_datetime,
                        help="start time of the schedule to compute")
    parser.add_argument("-d", "--delay", default=60, type=float,
                        help="delay (in seconds) needed between two "
                        + "consecutive passes (60 seconds by default)")
    parser.add_argument("-c", "--config", help="configuration file to use",
                        default=None)
    parser.add_argument("-o", "--output-dir",
                        help="where to put generated plots",
                        default=None)
    group = parser.add_argument_group(title="output")
    group.add_argument("-x", "--xml",
                       help="generate an xml request file and"
                       " put it in this directory. Could be a url",
                       default=None)
    group.add_argument("-r", "--report",
                       help="generate an xml report file and"
                       " put it in this directory. Could be a url",
                       default=None)
    group.add_argument("--scisys", default=None,
                       help="path to the schedule file")

    opts = parser.parse_args()

    if opts.config:
        coords, scores, station, area, forward, start = read_config(
            opts.config)

    if (not opts.config) and (not (opts.lon or opts.lat or opts.alt)):
        parser.error("Coordinates must be provided in the absence of "
                     "configuration file.")

    if not (opts.xml or opts.scisys or opts.report):
        parser.error("No output specified, use '--scisys' or '-x/--xml'")

    if opts.log:
        previous = os.path.exists(opts.log)
        handler = logging.handlers.RotatingFileHandler(opts.log,
                                                       backupCount=7)
        if previous:
            handler.doRollover()
    else:
        handler = logging.StreamHandler()
    handler.setFormatter(logging.Formatter("[%(levelname)s: %(asctime)s :"
                                           " %(name)s] %(message)s",
                                           '%Y-%m-%d %H:%M:%S'))
    if opts.verbose:
        loglevel = logging.DEBUG
    else:
        loglevel = logging.INFO

    handler.setLevel(loglevel)
    logging.getLogger('').setLevel(loglevel)
    logging.getLogger('').addHandler(handler)

    if opts.mail:
        mhandler = logging.handlers.SMTPHandler("localhost",
                                                "martin.raspaud@smhi.se",
                                                opts.mail,
                                                "Scheduler")
        mhandler.setLevel(logging.WARNING)
        logging.getLogger('').addHandler(mhandler)

    logger = logging.getLogger("trollsched")

    if opts.lon and opts.lat and opts.alt:
        coords = (opts.lon, opts.lat, opts.alt)

    # test line
    # python schedule.py -v 16.148649 58.581844 0.052765 -f 216 -s
    # 20140118140000 -t tle_20140120.txt -x . --scisys myched.txt

    satellites = scores.keys()

    # satellites = ["noaa 19", "noaa 18", "noaa 16", "noaa 15",
    #               "metop-a", "metop-b",
    #               "terra", "aqua",
    #               "suomi npp"]

    logger.info("Computing next satellite passes")

    tle_file = opts.tle
    if opts.forward:
        forward = opts.forward
    if opts.start_time:
        start_time = opts.start_time
    else:
        start_time = datetime.utcnow()  # + timedelta(hours=start)
    allpasses = get_next_passes(satellites, start_time,
                                forward, coords, tle_file)

    logger.info("Computation of next overpasses done")

    logger.debug(str(sorted(allpasses, key=lambda x: x.risetime)))

    area_boundary = AreaDefBoundary(area, frequency=500)
    area.poly = area_boundary.contour_poly

    logger.info("computing best schedule for area euron1")
    schedule, (graph, labels) = get_best_sched(allpasses, area, scores,
                                               timedelta(seconds=opts.delay))

    logger.debug(pformat(schedule))
    for opass in schedule:
        opass.rec = True
    logger.info("generating file")

    if opts.scisys:
        generate_sch_file(opts.scisys, allpasses, coords)

    if opts.output_dir is not None:
        for passage in allpasses:
            passage.save_fig(area.poly, directory=opts.output_dir)

    if opts.xml or opts.report:
        url = urlparse.urlparse(opts.xml or opts.report)
        if url.scheme not in ["file", ""]:
            directory = "/tmp"
        else:
            directory = url.path
        xmlfile = generate_xml_file(allpasses, start_time,
                                    start_time + timedelta(hours=forward),
                                    directory, station,
                                    opts.report)
        logger.info("Generated " + str(xmlfile))
        pathname, filename = os.path.split(xmlfile)
        del pathname
        if url.scheme in ["file", ""]:
            pass
        elif url.scheme == "ftp":
            import ftplib
            session = ftplib.FTP(url.hostname, url.username, url.password)
            with open(xmlfile, "rb") as xfile:
                session.storbinary('STOR ' + str(filename), xfile)
            session.quit()
        else:
            logger.error("Cannot save to " + str(url.scheme)
                         + ", but file is there" + str(xmlfile))

    if opts.graph is not None:
        now = datetime.now()
        graph.save("graph" + now.isoformat())
        graph.export(labels=[str(label) for label in labels],
                     filename=os.path.join(opts.graph,
                                           "sched" + now.isoformat() + ".gv"))

if __name__ == '__main__':
    try:
        run()
    except:
        logger.exception("Something wrong happened!")
        raise



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2014 Martin Raspaud

# Author(s):

#   Martin Raspaud <martin.raspaud@smhi.se>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Graph manipulation.
"""
import numpy as np

class Graph(object):

    def __init__(self, n_vertices=None, adj_matrix=None):
        if n_vertices is not None:
            self.order = n_vertices
            self.vertices = np.arange(self.order)
            self.adj_matrix = np.zeros((self.order, self.order), np.bool)
            self.weight_matrix = np.zeros((self.order, self.order), np.float)
        elif adj_matrix is not None:
            self.order = adj_matrix.shape[0]
            self.vertices = np.arange(self.order)
            self.adj_matrix = adj_matrix
            self.weight_matrix = np.zeros_like(adj_matrix)

    def weight(self, u, v):
        """weight of the *u*-*v* edge.
        """
        return self.weight_matrix[u, v]

    def neighbours(self, v):
        return self.vertices[self.adj_matrix[v, :] != 0]

    def add_edge(self, v1, v2, weight=1):
        self.weight_matrix[v1, v2] = weight
        self.weight_matrix[v2, v1] = weight
        self.adj_matrix[v1, v2] = True
        self.adj_matrix[v2, v1] = True

    def add_arc(self, v1, v2, weight=1):
        self.adj_matrix[v1, v2] = True
        self.weight_matrix[v1, v2] = weight

    def bron_kerbosch(self, r, p, x):
        """Get the maximal cliques.
        """
        if len(p) == 0 and len(x) == 0:
            yield r
        for v in p:
            for res in self.bron_kerbosch(r | set((v, )),
                                          p & set(self.neighbours(v)),
                                          x & set(self.neighbours(v))):
                yield res
            p = p - set((v, ))
            x = x | set((v, ))

    def dag_longest_path(self, v1, v2=None):
        """Give the longest path from *v1* to all other vertices or *v2* if
        specified. Assumes the vertices are sorted topologically and that the
        graph is directed and acyclic (DAG).
        """
        self.weight_matrix = -self.weight_matrix
        dist, path = self.dag_shortest_path(v1, v2)
        self.weight_matrix = -self.weight_matrix
        return dist, path

    def dag_shortest_path(self, v1, v2=None):
        """Give the sortest path from *v1* to all other vertices or *v2* if
        specified. Assumes the vertices are sorted topologically and that the
        graph is directed and acyclic (DAG). *v1* and *v2* are the indices of
        the vertices in the vertice list.
        """
        # Dijkstra for DAGs.

        dists = [np.inf] * self.order
        paths = [list() for _ in range(self.order)]

        dists[v1] = 0

        for u in self.vertices:
            # could be interrupted when we reach v2 ?
            for v in self.neighbours(u):
                if (dists[v] > dists[u] + self.weight(u, v)):
                    dists[v] = dists[u] + self.weight(u, v)
                    paths[v] = u

        if v2 is None:
            return dists, paths
        else:
            end = v2
            path = [end]
            while end != v1:
                path.append(paths[end])
                end = paths[end]
            return dists[v2], path

    def save(self, filename):
        np.savez_compressed(filename,
                            adj=self.adj_matrix,
                            weights=self.weight_matrix)

    def load(self, filename):
        stuff = np.load(filename)
        self.adj_matrix = stuff["adj"]
        self.weight_matrix = stuff["weights"]
        self.order = self.adj_matrix.shape[0]
        self.vertices = np.arange(self.order)

    def export(self, filename="./sched.gv", labels=None):
        """dot sched.gv -Tpdf -otruc.pdf
        """
        with open(filename, "w") as fd_:
            fd_.write("digraph schedule { \n size=\"80, 10\";\n center=\"1\";\n")
            for v1 in range(1, self.order - 1):
                for v2 in range(1, self.order - 1):
                    if self.adj_matrix[v1, v2]:
                        fd_.write('"' + str(labels[v1 - 1]) + '"' + " -> " +
                                  '"' + str(labels[v2 - 1]) + '"' +
                                  ' [ label = "' + str(self.weight_matrix[v1, v2]) + '" ];\n')

            fd_.write("}\n")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2014 Martin Raspaud

# Author(s):

#   Martin Raspaud <martin.raspaud@smhi.se>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
"""
# workaround python bug: http://bugs.python.org/issue15881#msg170215
import multiprocessing

from setuptools import setup
import imp
import sys

version = imp.load_source('trollsched.version', 'trollsched/version.py')

requires = ['numpy', 'pyresample', 'pyorbital']

if sys.version_info < (2, 7):
    # multiprocessing is not in the standard library
    requires.append('argparse')

setup(name='pytroll-schedule',
      version=version.__version__,
      description='Scheduling satellite passes in Python',
      author='Martin Raspaud',
      author_email='martin.raspaud@smhi.se',
      classifiers=["Development Status :: 4 - Beta",
                   "Intended Audience :: Science/Research",
                   "License :: OSI Approved :: GNU General Public License v3 " +
                   "or later (GPLv3+)",
                   "Operating System :: OS Independent",
                   "Programming Language :: Python",
                   "Topic :: Scientific/Engineering",
                   "Topic :: Scientific/Engineering :: Astronomy"],
      test_suite='trollsched.tests.suite',
      entry_points={
          'console_scripts': ['schedule = trollsched.schedule:run',
                              'compare_scheds = trollsched.compare:run']},
      packages=['trollsched'],
      install_requires=requires,
      zip_safe=False,
      )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2014 Martin Raspaud

# Author(s):

#   Martin Raspaud <martin.raspaud@smhi.se>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""The Boundary classes.
"""


import logging
import logging.handlers
from trollsched.spherical import SphPolygon
import numpy as np
from pyorbital import geoloc, geoloc_instrument_definitions


logger = logging.getLogger(__name__)


class Boundary(object):

    """Boundary objects.
    """

    def __init__(self, lons=None, lats=None, frequency=1):
        self._contour_poly = None
        if lons is not None:
            self.lons = lons[::frequency]
        if lats is not None:
            self.lats = lats[::frequency]

    def contour(self):
        return self.lons, self.lats

    @property
    def contour_poly(self):
        """Get the Spherical polygon corresponding to the Boundary
        """
        if self._contour_poly is None:
            self._contour_poly = SphPolygon(
                np.deg2rad(np.vstack(self.contour()).T))
        return self._contour_poly

    def draw(self, mapper, options):
        """Draw the current boundary on the *mapper*
        """
        self.contour_poly.draw(mapper, options)


class AreaBoundary(Boundary):

    """Area boundary objects.
    """

    def __init__(self, *sides):
        Boundary.__init__(self)
        self.sides_lons, self.sides_lats = zip(*sides)
        self.sides_lons = list(self.sides_lons)
        self.sides_lats = list(self.sides_lats)

    def decimate(self, ratio):
        """Remove some points in the boundaries, but never the corners.
        """
        for i in range(len(self.sides_lons)):
            l = len(self.sides_lons[i])
            start = (l % ratio) / 2
            points = np.concatenate(([0], np.arange(start, l, ratio), [l - 1]))

            self.sides_lons[i] = self.sides_lons[i][points]
            self.sides_lats[i] = self.sides_lats[i][points]

    def contour(self):
        """Get the (lons, lats) tuple of the boundary object.
        """
        lons = np.concatenate([lns[:-1] for lns in self.sides_lons])
        lats = np.concatenate([lts[:-1] for lts in self.sides_lats])

        return lons, lats


class AreaDefBoundary(AreaBoundary):

    """Boundaries for area definitions (pyresample)
    """

    def __init__(self, area, frequency=1):
        lons, lats = area.get_boundary_lonlats()
        AreaBoundary.__init__(self,
                              (lons.side1, lats.side1),
                              (lons.side2, lats.side2),
                              (lons.side3, lats.side3),
                              (lons.side4, lats.side4))

        if frequency != 1:
            self.decimate(frequency)


class SwathBoundary(Boundary):

    """Boundaries for satellite overpasses.
    """

    def get_instrument_points(self, overpass, utctime,
                              scans_nb, scanpoints, frequency=1):
        """Get the boundary points for a given overpass.
        """
        instrument = overpass.instrument
        # cheating at the moment.
        scan_angle = 55.37
        if instrument == "modis":
            scan_angle = 55.0
        elif instrument == "viirs":
            scan_angle = 55.84
        elif overpass.satellite == "noaa 16":
            scan_angle = 55.25
        instrument = "avhrr"
        instrument_fun = getattr(geoloc_instrument_definitions, instrument)
        sgeom = instrument_fun(scans_nb, scanpoints,
                               scan_angle=scan_angle, frequency=frequency)
        times = sgeom.times(utctime)
        pixel_pos = geoloc.compute_pixels((self.orb.tle._line1,
                                           self.orb.tle._line2),
                                          sgeom, times)
        lons, lats, alts = geoloc.get_lonlatalt(pixel_pos, times)

        del alts
        return (lons.reshape(-1, len(scanpoints)),
                lats.reshape(-1, len(scanpoints)))

    def __init__(self, overpass, frequency=100.0):
        # compute area covered by pass

        Boundary.__init__(self)

        self.overpass = overpass
        self.orb = overpass.orb

        # compute sides

        scans_nb = np.ceil(((overpass.falltime - overpass.risetime).seconds +
                            (overpass.falltime -
                             overpass.risetime).microseconds
                            / 1000000.0) / frequency)

        sides_lons, sides_lats = self.get_instrument_points(self.overpass,
                                                            overpass.risetime,
                                                            scans_nb,
                                                            np.array(
                                                                [0, 2047]),
                                                            frequency=frequency)

        self.left_lons = sides_lons[::-1, 0]
        self.left_lats = sides_lats[::-1, 0]
        self.right_lons = sides_lons[:, 1]
        self.right_lats = sides_lats[:, 1]

        # compute bottom

        # avhrr
        maxval = 2048
        rest = maxval % frequency
        reduced = np.hstack(
            [0, np.arange(rest / 2, maxval, frequency), maxval - 1])

        lons, lats = self.get_instrument_points(self.overpass,
                                                overpass.falltime,
                                                1,
                                                reduced)

        self.bottom_lons = lons[0][::-1]
        self.bottom_lats = lats[0][::-1]

        # compute top

        lons, lats = self.get_instrument_points(self.overpass,
                                                overpass.risetime,
                                                1,
                                                reduced)

        self.top_lons = lons[0]
        self.top_lats = lats[0]

    def decimate(self, ratio):
        l = len(self.top_lons)
        start = (l % ratio) / 2
        points = np.concatenate(([0], np.arange(start, l, ratio), [l - 1]))

        self.top_lons = self.top_lons[points]
        self.top_lats = self.top_lats[points]
        self.bottom_lons = self.bottom_lons[points]
        self.bottom_lats = self.bottom_lats[points]

        l = len(self.right_lons)
        start = (l % ratio) / 2
        points = np.concatenate(([0], np.arange(start, l, ratio), [l - 1]))

        self.right_lons = self.right_lons[points]
        self.right_lats = self.right_lats[points]
        self.left_lons = self.left_lons[points]
        self.left_lats = self.left_lats[points]

    def contour(self):
        lons = np.concatenate((self.top_lons,
                               self.right_lons[1:-1],
                               self.bottom_lons,
                               self.left_lons[1:-1]))
        lats = np.concatenate((self.top_lats,
                               self.right_lats[1:-1],
                               self.bottom_lats,
                               self.left_lats[1:-1]))
        return lons, lats
